@@ -2470,8 +2470,17 @@ glmer_bhat(double pars[], GlmerStruct GS)
     Free(etaold);
 }
 
+/** 
+ * Evaluate the difference in the quadratic form in the
+ * random effects and the delb's.
+ * 
+ * @param GS a GlmerStruct object
+ * @param b random effects
+ * 
+ * @return the quadratic form
+ */
 static double
-b_quadratic_form(GlmerStruct GS, SEXP b) {
+b_qf(GlmerStruct GS, SEXP b, SEXP delb) {
     SEXP Omega = GET_SLOT(GS->mer, Matrix_OmegaSym);
     int *nc = INTEGER(GET_SLOT(GS->mer, Matrix_ncSym)),
 	*Gp = INTEGER(GET_SLOT(GS->mer, Matrix_GpSym)),
@@ -2488,36 +2497,110 @@ b_quadratic_form(GlmerStruct GS, SEXP b) {
 			REAL(VECTOR_ELT(Omega, i)), &nci,
 			&zero, tmp, &nlev);
         ans += F77_CALL(ddot)(&ntot, tmp, &ione, tmp, &ione);
+	if (delb != R_NilValue) {
+	    double *delbi = REAL(VECTOR_ELT(delb, i));
+	    ans -= F77_CALL(ddot)(&ntot, delbi, &ione, delbi, &ione);
+	}
     }
     return ans;
 }
 	
+/** 
+ * Evaluate the conditional deviance, given a value of the random
+ * effects
+ * 
+ * @param GS a GlmerStruct object
+ * @param b random effects
+ * 
+ * @return the conditional deviance
+ */
 static double
-marginal_deviance(GlmerStruct GS) {
-    SEXP devr = PROTECT(eval_check(GS->dev_resids, GS->rho,
-				   REALSXP, GS->n));
+cond_dev(GlmerStruct GS, SEXP b) {
+    SEXP devr;
     int i;
     double ans = 0;
-    
-    for (i = 0; i < GS->n; i++) ans += REAL(devr)[i];
+
+    fitted_ranef(GET_SLOT(GS->mer, Matrix_flistSym), GS->unwtd, b,
+		 INTEGER(GET_SLOT(GS->mer, Matrix_ncSym)),
+		 Memcpy(REAL(GS->eta), REAL(GS->off), GS->n));
+    eval_check_store(GS->linkinv, GS->rho, GS->mu);
+    devr = PROTECT(eval_check(GS->dev_resids, GS->rho,
+			      REALSXP, GS->n));    
+    for (i = 0, ans = 0; i < GS->n; i++) ans += REAL(devr)[i];
     UNPROTECT(1);
     return ans;
 }
 
+/** 
+ * Evaluate the deviance at b/delb relative to that at bhat/0
+ * 
+ * @param GS a GlmerStruct object
+ * @param b random effects
+ * @param delb change from bhat on the delb scale
+ * @param cnst additive constant
+ * 
+ * @return the relative deviance
+ */
+static R_INLINE double
+relDev(GlmerStruct GS, SEXP b, SEXP delb, double cnst)
+{
+    return cond_dev(GS, b) + b_qf(GS, b, delb) - cnst;
+}
+	
+static void
+update_delb_b(double x, int nc, int nlev, const double bvFac[],
+	      const double bhat[], double delb[], double bt[])
+{
+    int i;
+    if (nc != 1) error("code not yet written");
+    for (i = 0; i < nlev; i++)
+	bt[i] = bhat[i] + (delb[i] = x * bvFac[i]);
+}
+
 SEXP glmer_devAGQ(SEXP pars, SEXP GSp, SEXP nAGQp)
 {
+    SEXP bhat;
     GlmerStruct GS = (GlmerStruct) R_ExternalPtrAddr(GSp);
+    double bvd, cnst, deviance, rmlik;
     int nAGQ = asInteger(nAGQp);
 	
     if (!isReal(pars) || LENGTH(pars) != GS->npar)
 	error(_("`%s' must be a numeric vector of length %d"),
 	      "pars", GS->npar);
     glmer_bhat(REAL(pars), GS);
-    if (nAGQ > 1) error("code not yet written");
+    bvd = -2 * Sigma_bVar_det(GS); /* also factors Omega and bVar */
+    bhat = PROTECT(lmer_ranef(GS->mer));
+    deviance = cnst = cond_dev(GS, bhat) + b_qf(GS, bhat, R_NilValue);
+    rmlik = 1;			/* relative marginal likelihood */
+    if (nAGQ > 1) {
+	SEXP delb = PROTECT(duplicate(bhat)), btrial = PROTECT(duplicate(bhat));
+	SEXP bvFac, delb0, btrial0, bhat0;
+	int *dims, i, odd = nAGQ % 2;
+	int n2 = (nAGQ + odd)/2;
 
-    return ScalarReal(marginal_deviance(GS)
-		      - 2 * Sigma_bVar_det(GS)
-		      - b_quadratic_form(GS, lmer_ranef(GS->mer)));
+	rmlik = 0;
+	if (GS->nf > 1)
+	    error(_("AGQ available only for a single grouping factor"));
+	bvFac = VECTOR_ELT(GET_SLOT(GS->mer, Matrix_bVarSym), 0);
+	delb0 = VECTOR_ELT(delb, 0);
+	bhat0 = VECTOR_ELT(bhat, 0);
+	btrial0 = VECTOR_ELT(btrial, 0);
+	dims = INTEGER(getAttrib(bhat0, R_DimSymbol));
+	for (i = 0; i < n2; i++) {
+	    update_delb_b(GHQ_x[nAGQ][i], dims[1], dims[0], REAL(bvFac),
+			  REAL(bhat0), REAL(delb0), REAL(btrial0));
+	    rmlik += GHQ_w[nAGQ][i] * exp(-relDev(GS, btrial, delb, cnst)/2);
+	    if (GS->EMverbose) Rprintf("%10g ", rmlik);
+	}
+	for (i = (n2 - 1) - odd; i >= 0; i--) {
+	    update_delb_b(-GHQ_x[nAGQ][i], dims[1], dims[0], REAL(bvFac),
+			  REAL(bhat0), REAL(delb0), REAL(btrial0));
+	    rmlik += GHQ_w[nAGQ][i] * exp(-relDev(GS, btrial, delb, cnst)/2);
+	    if (GS->EMverbose) Rprintf("%10g ", rmlik);
+	}
+	if (GS->EMverbose) Rprintf("\n", rmlik);
+	UNPROTECT(2);
+    }
+    UNPROTECT(1);
+    return ScalarReal(deviance - 2*log(rmlik) + bvd);
 }
-
-
